'#Reference {00020813-0000-0000-C000-000000000046}#1.7#0#C:\Program Files (x86)\Microsoft Office\Office14\EXCEL.EXE#Microsoft Excel 14.0 Object Library#Excel
Sub Main

''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'File: gr2excel.BAS
'Version 01: Build Release
'Date: 5/13/2016
'Author: Ryan Blelloch, ATA Engineering
'Support: plm_sales@ata-e.com
'www.ata-plmsoftware.com
'
'Takes node and element data of user-selected groups and prints it to an Excel sheet
'
'Copyright (c) <2020> <ATA Engineering, Inc.>
'
'Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
'associated documentation files (the "Software") to use, copy, modify, merge, and/or distribute
'royalty-free copies of the Software, and to permit persons to whom the Software is furnished
'to do so, subject to the following conditions:
'The above copyright notice and this permission notice shall be included in all copies or
'substantial portions of the Software.
'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
'INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
'OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''

	'Connects program with the model currently open
    Dim app As femap.model
    Set app = feFemap()

    'This connects with the excel application you referenced with the first line
    Dim appExcel As Excel.Application
    Set appExcel =  New Excel.Application

    'These variables will be used to control the new workbook opened
    Dim wbk As Excel.Workbook
    Dim wks As Excel.Worksheet

    'This creates the new workbook to be used
    Set wbk = appExcel.Workbooks.Add

    'This connects to the first worksheet of the new workbook
    Set wks = wbk.Worksheets(1)

    'fill in data
    getGroupInfo( app, wks )

    'Build headers
    fillHeaderInfo( wks )

    'Show excel sheet
    wks.Columns.AutoFit
    appExcel.Visible = True
    
End Sub

Public Sub getGroupInfo( ByRef app As femap.model, ByRef wks As Excel.Worksheet )

	'Get groups from user
    Dim allGroups As femap.Set
    Set allGroups = app.feSet

    Dim curGroup As femap.Group
    Set curGroup = app.feGroup

    Dim sortIDs As Variant
    Set sortIDs = app.feSort

    Dim setIDs As femap.Set
    Set setIDs = app.feSet

    Dim dTable As femap.DataTable
    Set dTable = app.feDataTable
    dTable.Clear

    Dim elemSet As femap.Set
    Set elemSet = app.feSet

    Dim length As Double
    Dim area As Double
    Dim vol As Double
    Dim tMass As Double

    Dim tCG As Variant
    Dim sMass As Double
    Dim nMass As Double
    Dim sCG As Variant
    Dim nCG As Variant
    Dim inert As Variant
    Dim inertCG As Variant
    Dim result As Long

    Dim noNodes, noElems As Boolean

    rc = allGroups.Select(FT_GROUP, True, "Select Groups to grab information")

    Dim curRow As Integer
    curRow = 2

    While allGroups.Next

    	curGroup.Get( allGroups.CurrentID )

    	app.feAppMessage( FCM_NORMAL, "Working on group " + curGroup.title )

    	noNodes = False
    	noElems = False

    	wks.Cells( curRow, 1 ) = curGroup.ID
    	wks.Cells( curRow, 2 ) = curGroup.title
    	result = getMinNode( app, curGroup.ID )
    	If (result > 0) Then
    		wks.Cells( curRow, 3 ) = result
    	End If
    	result = getMaxNode( app, curGroup.ID )
    	If (result > 0) Then
    		wks.Cells( curRow, 4 ) = result
    	End If
    	result = getNumNodes( app, curGroup.ID )
    	wks.Cells( curRow, 5 ) = result
    	If (result = 0) Then
    		noNodes = True
    	End If
    	result = getMinElem( app, curGroup.ID )
    	If (result > 0) Then
    		wks.Cells( curRow, 6 ) = result
    	End If
    	result = getMaxElem( app, curGroup.ID )
    	If (result > 0) Then
    		wks.Cells( curRow, 7 ) = result
    	End If
    	result = getNumElems( app, curGroup.ID )
    	wks.Cells( curRow, 8 ) = result
    	If (result = 0) Then
    		noElems = True
    	End If
    	elemSet.Clear
    	elemSet.AddGroup( FT_ELEM, curGroup.ID )
    	rc = app.feMeasureMeshMassProp( elemSet.ID, 0, False, False, length, area, vol, sMass, nMass, tMass, sCG, nCG, tCG, inert, inertCG )
    	wks.Cells( curRow, 9 ) = tMass
    	wks.Cells( curRow, 10 ) = tCG(0)
    	wks.Cells( curRow, 11 ) = tCG(1)
    	wks.Cells( curRow, 12 ) = tCG(2)
    	wks.Cells( curRow, 13 ) = length
    	wks.Cells( curRow, 14 ) = area
    	wks.Cells( curRow, 15 ) = vol

    	If ( (noElems) And (noNodes) ) Then
    		wks.Cells( curRow, 1 ).EntireRow.Hidden = True
    		app.feAppMessage( FCM_HIGHLIGHT, "Hiding " + curGroup.title + " (no nodes or elems)" )
    	End If

    	curRow = curRow + 1

    Wend

End Sub

Public Sub fillHeaderInfo( ByRef wks As Excel.Worksheet )

	wks.Cells( 1, 1 ) = "Group ID"
	wks.Cells( 1, 2 ) = "Group Name"
	wks.Cells( 1, 3 ) = "Lowest Node ID"
	wks.Cells( 1, 4 ) = "Highest Node ID"
	wks.Cells( 1, 5 ) = "# of Nodes"
	wks.Cells( 1, 6 ) = "Lowest Element ID"
	wks.Cells( 1, 7 ) = "Highest Element ID"
	wks.Cells( 1, 8 ) = "# of Elements"
	wks.Cells( 1, 9 ) = "Mass"
	wks.Cells( 1, 10 ) = "CG (x)"
	wks.Cells( 1, 11 ) = "CG (y)"
	wks.Cells( 1, 12 ) = "CG (z)"
	wks.Cells( 1, 13 ) = "Length"
	wks.Cells( 1, 14 ) = "Area"
	wks.Cells( 1, 15 ) = "Volume"

	wks.Range( wks.Cells( 1, 1 ) , wks.Cells( 1, 15 ) ).Font.Bold = True

End Sub

Public Function getMinNode( ByRef app As femap.model, curGroup As Integer ) As Long
	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_NODE, curGroup )
	getMinNode = nd.First

End Function

Public Function getMaxNode( ByRef app As femap.model, curGroup As Integer ) As Long
	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_NODE, curGroup )
	getMaxNode = nd.Last

End Function

Public Function getNumNodes( ByRef app As femap.model, curGroup As Integer ) As Long

	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_NODE, curGroup )
	getNumNodes = nd.Count

End Function

Public Function getMinElem( ByRef app As femap.model, curGroup As Integer ) As Long
	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_ELEM, curGroup )
	getMinElem = nd.First

End Function

Public Function getMaxElem( ByRef app As femap.model, curGroup As Integer ) As Long
	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_ELEM, curGroup )
	getMaxElem = nd.Last

End Function

Public Function getNumElems( ByRef app As femap.model, curGroup As Integer ) As Long

	Dim nd As femap.Set
	Set nd = app.feSet

	nd.AddGroup( FT_ELEM, curGroup )
	getNumElems = nd.Count

End Function
